from __future__ import annotations

from typing import Any

import ckan.plugins as plugins
import ckan.plugins.toolkit as toolkit
from ckan import model

from . import cli, helpers, views
from .logic import action, auth
from .model import Report

CONFIG_CASCADE_DELETE = "ckanext.check_link.remove_reports_when_resource_deleted"


class CheckLinkPlugin(plugins.SingletonPlugin):
    plugins.implements(plugins.IConfigurer)
    plugins.implements(plugins.IActions)
    plugins.implements(plugins.IAuthFunctions)
    plugins.implements(plugins.IBlueprint)
    plugins.implements(plugins.IClick)
    plugins.implements(plugins.ITemplateHelpers)
    plugins.implements(plugins.IDomainObjectModification, inherit=True)

    def notify(self, entity: Any, operation: str) -> None:
        if isinstance(entity, model.Resource) and entity.state == "deleted":
            if toolkit.asbool(toolkit.config.get(CONFIG_CASCADE_DELETE)):
                _remove_resource_report(entity.id)

    # ITemplateHelpers

    def get_helpers(self):
        return helpers.get_helpers()

    # IConfigurer
    def update_config(self, config_):
        toolkit.add_template_directory(config_, "templates")
        toolkit.add_public_directory(config_, "public")
        toolkit.add_resource("assets", "check_link")

    # IActions
    def get_actions(self):
        return action.get_actions()

    # IAuthFunctions
    def get_auth_functions(self):
        return auth.get_auth_functions()

    # IBlueprint
    def get_blueprint(self):
        return views.get_blueprints()

    # IClick
    def get_commands(self):
        return cli.get_commands()


def _remove_resource_report(resource_id: str):
    report = Report.by_resource_id(resource_id)
    if report:
        model.Session.delete(report)



"""add_ondelete_cascade_to_resource_id

Revision ID: dad15b4f251a
Revises: 564f2016af51
Create Date: 2023-07-20 18:53:22.944540

"""
from alembic import op

# revision identifiers, used by Alembic.
revision = "dad15b4f251a"
down_revision = "564f2016af51"
branch_labels = None
depends_on = None


def upgrade():
    op.drop_constraint("check_link_report_resource_id_fkey", "check_link_report")
    op.create_foreign_key(
        "check_link_report_resource_id_fkey",
        "check_link_report",
        "resource",
        ["resource_id"],
        ["id"],
        ondelete="CASCADE",
    )


def downgrade():
    op.drop_constraint("check_link_report_resource_id_fkey", "check_link_report")
    op.create_foreign_key(
        "check_link_report_resource_id_fkey",
        "check_link_report",
        "resource",
        ["resource_id"],
        ["id"],
    )



import pytest

from ckan.plugins import plugin_loaded
from ckan.tests.helpers import call_action

from ckanext.check_link.model import Report


@pytest.mark.ckan_config("ckan.plugins", "check_link")
@pytest.mark.usefixtures("with_plugins")
def test_plugin():
    assert plugin_loaded("check_link")


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestCascadeReportRemoval:
    def test_not_removed_by_default(self, report):
        call_action("resource_delete", id=report["resource_id"])
        assert Report.by_resource_id(report["resource_id"])

    @pytest.mark.ckan_config(
        "ckanext.check_link.remove_reports_when_resource_deleted", "true"
    )
    def test_removed_if_configured(self, report):
        call_action("resource_delete", id=report["resource_id"])
        assert not Report.by_resource_id(report["resource_id"])

    @pytest.mark.ckan_config(
        "ckanext.check_link.remove_reports_when_resource_deleted", "true"
    )
    def test_removed_if_resource_dropped_via_package_update(self, report):
        resource = call_action("resource_show", id=report["resource_id"])
        call_action("package_patch", id=resource["package_id"], resources=[])

        assert not Report.by_resource_id(report["resource_id"])
